<?php 
/**
 * Library functions for usercategories
 *
 * @author  Alicia García Holgado  aliciagh@usal.es
 * @version 1.0
 * @license http://www.gnu.org/copyleft/gpl.html GNU Public License
 * @package mod/usercategories
 */
require_once($CFG->dirroot.'/course/lib.php');

define('USERCATEGORIES', $CFG->prefix.'usercategories');
define('USERCATEGORIES_COURSE', $CFG->prefix.'usercategories_course');

/**
 * Execute post-install custom actions for the module
 * 
 * @return boolean true if success, false on error
 */
function usercategories_install() {
    global $CFG;
    
    if ($module = get_record("modules", "name", "usercategories")) {
	    set_field("modules", "visible", "0", "id", $module->id); // Hide main module
	    // Remember the visibility status in visibleold
	    // and hide...
	    $sql = "UPDATE {$CFG->prefix}course_modules SET visibleold=visible, visible=0 WHERE  module={$module->id}";
	    execute_sql($sql, false);
	    // clear the course modinfo cache for courses
	    // where we just deleted something
	    $sql = "UPDATE {$CFG->prefix}course SET modinfo='' WHERE id IN (SELECT DISTINCT course FROM {$CFG->prefix}course_modules WHERE visibleold=1 AND module={$module->id})";
	    execute_sql($sql, false);
	    
	    return true;
    } else {
        return false;
    }
}

/**
 * Recursive function to print all the categories ready for editing.
 */
function print_category_edit() {

    global $CFG, $USER;

    static $str = '';

    if (empty($str)) {
        $str->edit     = get_string('edit');
        $str->delete   = get_string('delete');
        $str->edit     = get_string('editthiscategory');
    }

    echo '<table class="generalbox editcourse boxaligncenter" cellspacing="1" cellpadding="5"><tr class="header">';
    echo '<th class="header" scope="col">'.get_string('categories').'</th>';
    echo '<th class="header" scope="col">'.get_string('courses').'</th>';
    echo '<th class="header" scope="col">'.get_string('edit').'</th>';
    echo '</tr>';
    
    $categories = get_records_sql('select * from '.USERCATEGORIES.' where userid='.$USER->id.' order by ordercat');
    if(!empty($categories)) {
        foreach($categories as $category) {
            echo '<tr><td align="left" class="name">';
            
            echo '<a title="'.$category->name.'" '.
                 ' href="category.php?id='.$category->id.'">'.
                 format_string($category->name).'</a>';
            echo '</td>';
            
            $coursecount = get_record_sql('select count(*) as c from '.USERCATEGORIES_COURSE.' where categoryid='.$category->id.' group by categoryid');
            if(empty($coursecount)) {
                $coursecount->c = 0;
            }
            echo '<td align="right" class="count">'.$coursecount->c.'</td>';
    
            echo '<td align="right" class="icons">';    /// Print little icons
    
            echo '<a title="'.$str->edit.'" href="edit_category.php?id='.$category->id.'"><img'.
                     ' src="'.$CFG->pixpath.'/t/edit.gif" class="iconsmall" alt="'.$str->edit.'" /></a> ';
    
            echo '<a title="'.$str->delete.'" href="delete_category.php?delete='.$category->id.'"><img'.
                     ' src="'.$CFG->pixpath.'/t/delete.gif" class="iconsmall" alt="'.$str->delete.'" /></a> ';
            
            echo '</td>';
            echo '</tr>';
        }
    }
    
    echo '<tr><td align="left" class="name">';
        
    echo '<a title="'.get_string('nocategory', 'usercategories').'" '.' href="category.php?id=-1">'.get_string('nocategory', 'usercategories').'</a>';
    echo '</td>';
    
    $courses = get_my_courses($USER->id, 'visible DESC,fullname ASC', '*', false, 100);
    $coursecount = get_records_sql('select * from '.USERCATEGORIES_COURSE.' left join '.USERCATEGORIES.
                    ' on '.USERCATEGORIES.'.id='.USERCATEGORIES_COURSE.'.categoryid where '.USERCATEGORIES.'.userid='.$USER->id);
    if (empty($coursecount)) {
        $count = count($courses);
    } else {
        $count = count($courses) - count($coursecount);
    }
    echo '<td align="right" class="count">'.$count.'</td>';
    echo '</tr>';
    
    echo '</table>';
}

/**
 * Recursive function to print all course in one category.
 */
function print_category_list($cat) {
    
    global $USER, $CFG;
    
    echo '<table class="generalbox editcourse boxaligncenter" cellspacing="1" cellpadding="5"><tr class="header">';
    echo '<th class="header" scope="col">'.get_string('category').'</th>';
    echo '<th class="header" scope="col">'.get_string('courses').'</th>';
    echo '</tr>';
    
    $displaylist = make_usercategories_list();
    if(is_object($cat)) {
        $courses = get_records_sql('select * from '.USERCATEGORIES_COURSE.' left join '.USERCATEGORIES.
                    ' on '.USERCATEGORIES.'.id='.USERCATEGORIES_COURSE.'.categoryid where '.USERCATEGORIES.'.userid='.$USER->id.' and '.USERCATEGORIES.'.id='.$cat->id);
        if(!empty($courses)) {
            foreach($courses as $course) {
                $c = get_record('course', 'id', $course->courseid);
                echo '<tr><td align="left" class="name">';
                
                echo '<a title="'.format_string($c->fullname).'" href="'.$CFG->wwwroot.'/course/view.php?id='.$c->id.'">'.format_string($c->fullname).'</a>';
                echo '</td>';
                
                echo '<td align="right">';
                $catname = get_field('usercategories', 'name', 'id', $course->categoryid);
                popup_form('category.php?last='.$course->categoryid.'&amp;move='.$course->courseid.'&amp;catid=', $displaylist, 'switchusercat'.$course->courseid, $course->categoryid, '');
                echo '</td>';
                echo '</tr>';
            }
        }
    } else {
        $cou = get_my_courses($USER->id, 'visible DESC,fullname ASC', '*', false, 100);
        $courses = get_records_sql('select * from '.USERCATEGORIES_COURSE.' left join '.USERCATEGORIES.
                    ' on '.USERCATEGORIES.'.id='.USERCATEGORIES_COURSE.'.categoryid where '.USERCATEGORIES.'.userid='.$USER->id);
        if(!empty($cou)) {
            foreach($cou as $course) {
                $flag = false;
                
                if(!empty($courses)) {
                    foreach($courses as $cs) {
                        if($cs->courseid == $course->id) $flag = true;
                    }
                }
                if(!$flag) {
                    $c = get_record('course', 'id', $course->id);
                    echo '<tr><td align="left" class="name">';
                    
                    echo '<a title="'.format_string($c->fullname).'" href="'.$CFG->wwwroot.'/course/view.php?id='.$c->id.'">'.format_string($c->fullname).'</a>';
                    echo '</td>';
                    
                    echo '<td align="right">';
                    popup_form('category.php?last=-1&amp;move='.$course->id.'&amp;catid=', $displaylist, 'switchusercat'.$course->id, -1, '');
                    echo '</td>';
                    echo '</tr>';
                }
            }
        }
    }
    
    echo '</table>';
    
    echo '<div class="buttons" style="text-align:center">';
    print_single_button('category.php', '', get_string('back'), 'get');
    echo '</div>';
    
}

/**
 * List of user categories.
 * 
 * @return array Names of categories.
 */
function make_usercategories_list() {
    
    global $USER;
    
    $categories = get_records('usercategories', 'userid', $USER->id);
    
    $displaylist = array();
    foreach($categories as $category) {
        $displaylist[$category->id] = $category->name;
    }
    $displaylist[-1] = get_string('nocategory', 'usercategories');
    
    return $displaylist;
}

/**
 * Display my courses grouped by categories.
 */
function print_overview_usercategories($courses) {

    global $CFG, $USER;
    
    
    $htmlarray = array();
    if ($modules = get_records('modules')) {
        foreach ($modules as $mod) {
            if (file_exists(dirname(dirname(dirname(__FILE__))).'/mod/'.$mod->name.'/lib.php')) {
                include_once(dirname(dirname(dirname(__FILE__))).'/mod/'.$mod->name.'/lib.php');
                $fname = $mod->name.'_print_overview';
                if (function_exists($fname)) {
                    $fname($courses,$htmlarray);
                }
            }
        }
    }
    
    print_buttons();
    
    // Order courses by user categories.
    $categories = get_records_sql('select * from '.USERCATEGORIES.' where userid='.$USER->id.' order by ordercat ASC');
    $intercalar = true;
    $contador = 0;
    if(!empty($categories)) {
        foreach($categories as $category) {
            echo '<h2>'.$category->name.'</h2>';
            
            $temp = $contador;
            foreach ($courses as $course) {
                $cat = get_record_sql('select categoryid from '.USERCATEGORIES_COURSE.' left join '.USERCATEGORIES.
                                          ' on '.USERCATEGORIES.'.id='.USERCATEGORIES_COURSE.'.categoryid'.
                                              ' where '.USERCATEGORIES.'.userid='.$USER->id.' and '.USERCATEGORIES_COURSE.'.courseid='.$course->id);
                if(!empty($cat) && $cat->categoryid == $category->id) {
                    $contador++;
                    print_course_categories($course, $intercalar, $htmlarray);
                }
            }
            if($temp != $contador) $intercalar = $intercalar ? false : true;
        }
    }
    
    // Display courses without user category.
    if(count($courses) != $contador) {
        echo '<h2>'.get_string('nocategory', 'usercategories').'</h2>';
        
        foreach($courses as $course) {
            $cat = get_record_sql('select categoryid from '.USERCATEGORIES_COURSE.' left join '.USERCATEGORIES.
                                    ' on '.USERCATEGORIES.'.id='.USERCATEGORIES_COURSE.'.categoryid'.
                                        ' where '.USERCATEGORIES.'.userid='.$USER->id.' and '.USERCATEGORIES_COURSE.'.courseid='.$course->id);
            if(empty($cat)) {
                print_course_categories($course, $intercalar, $htmlarray);
            }
        } 
    }
}

/**
 * Display three buttons in the top of courses list that allow add new course, show or hide extra info and customize user categories.
 */
function print_buttons() {
    
    global $CFG, $USER;
    
    $confirm = optional_param('confirm', 0, PARAM_BOOL);
    $visible_info = optional_param('visible_info', PARAM_INT);
    
    echo '<table><tr valign="top">';
    
    // Print link to create a new course.
    if (has_capability('moodle/course:create', get_context_instance(CONTEXT_SYSTEM))) {
        // Get the 1st available category
        $options = array('category' => get_field('course_categories', 'id', 'parent', '0'));
        echo '<td valign="top">';
        print_single_button($CFG->wwwroot.'/course/edit.php', $options, get_string('addnewcourse'), 'get');
        echo '</td>';
    }
    
    // Check visible info preference.
    if(!get_record('user_preferences','name', 'visible_info_course', 'userid', $USER->id)) {
        execute_sql('insert into '.$CFG->prefix.'user_preferences (userid, name, value) values ('.$USER->id.', "visible_info_course", 1)', false);
    }
    if($confirm) {
        $wheresql = where_clause('userid', $USER->id, 'name', 'visible_info_course');
        set_field_select('user_preferences', 'value', $visible_info, $wheresql, true);
    }
    
    // Print button to show/hide extra info.
    echo '<td valign="top">';
    reload_user_preferences();
    if($USER->preference['visible_info_course']) {
        $options = array('visible_info' => 0, 'confirm' => 'yes');
        print_single_button($CFG->wwwroot.'/my/index.php', $options, get_string('hideinfo', 'usercategories'), 'post', '_self', false, get_string('hideinfomessage', 'usercategories'));
    } else {
        $options = array('visible_info' => 1, 'confirm' => 'yes');
        print_single_button($CFG->wwwroot.'/my/index.php', $options, get_string('showinfo', 'usercategories'), 'post', '_self', false, get_string('showinfomessage', 'usercategories'));
    }
    echo '</td>';
    
    // Print button to customize user categories.
    echo '<td valign="top">';
    print_single_button($CFG->wwwroot.'/mod/usercategories/category.php', '', get_string('custom', 'usercategories'), 'post');
    echo '</td>';
    
    echo '</tr></table>';
    echo '<br>';
}

/**
 * Display a course with delete button.
 */
function print_course_categories($course, $intercalar, $htmlarray) {
    
    global $CFG, $USER;
    
    if ($intercalar) {
        print_simple_box_start('center', '100%', '', 5, "coursebox");
    } else {
        print_simple_box_start('center', '100%', '', 5, "coursebox usercategorycourse");
    }

    $linkcss = '';
    if ( empty($course->visible)) {
        $linkcss = 'class="dimmed"';
    }
    
    $context = get_context_instance(CONTEXT_COURSE, $course->id);
    if (has_capability('moodle/role:unassignself', $context, NULL, false) and get_user_roles($context, $USER->id, false)) {
        echo '<a title="'.get_string('deletecourse', 'usercategories').'" href="'.$CFG->wwwroot.'/course/unenrol.php?id='.$course->id.'" style="float:right; margin: 6px">'.'<img src="'.$CFG->wwwroot.'/mod/usercategories/icon_delete.gif" /></a>';
    }
    
    print_heading('<a title="'.format_string($course->fullname).'" '.$linkcss.' href="'.$CFG->wwwroot.'/course/view.php?id='.$course->id.'">'.format_string($course->fullname).'</a>');
    
    if ($USER->preference['visible_info_course']) { // Check if show or hide extra info.
        if (array_key_exists($course->id, $htmlarray)) {
            foreach ($htmlarray[$course->id] as $modname=>$html) {
                echo $html;
            }
        }
    }
    print_simple_box_end();
}

?>